{-# LANGUAGE ScopedTypeVariables, RecordWildCards #-}
module Main
where

import System.IO (
  Handle,
  IOMode (WriteMode),
  openBinaryFile,
  stdout
  )
import Text.XML.HXT.Core

import CommandLine (Args(..), apply_args)
import LWN.Page

my_read :: String -> IOStateArrow s b XmlTree
my_read =
  readDocument [ withValidate  no,
                 withParseHTML yes,
                 withInputEncoding utf8,
                 withWarnings no ]

-- | If we're given an empty path, return a handle to
--   'stdout'. Otherwise, open the given file and return a read/write
--   handle to that.
get_output_handle :: FilePath -> IO Handle
get_output_handle path =
  if (null path) then
    return stdout
  else
    openBinaryFile path WriteMode


-- | Convert the given article to either a URL or a filesystem
--   path. If the given article exists on the filesystem, we assume
--   it's a file. Otherwise, we check to see if it's a URL. Failing
--   that, we try to construct a URL from what we're given and do our
--   best.
real_article_path :: String -> IO String
real_article_path = return . id

main :: IO ()
main = do
  Args{..} <- apply_args
  output_handle <- get_output_handle output
  input_path <- real_article_path article
  let html = my_read input_path
  result <- parse html

  case result of
    Just stuff  -> epublish stuff output_handle
    Nothing     -> return ()

  putStrLn "Done."
